/**
*
*/
import RTIcap.*;
import npacrti.*;

/*
*/
public class 
DeclarationManager extends _DeclarationManagerInterfaceImplBase
                   implements RTIDebug
{
    ObjectClassNaming objNaming ;
    InteractionNaming interNaming ;


    /**
    */
    public 
    DeclarationManager(ObjectClassNaming _objNaming, InteractionNaming _interNaming)
    { objNaming = _objNaming;
      interNaming = _interNaming;
    }

    /**
    * 5.2
    */
    public
    void publishObjectClass(short theClass, short[] attributeList,
                            RTIcap.FederateAmbassador _fedHandle, short _fedId   )
         throws RTIcap.ObjectClassNotDefined, RTIcap.AttributeNotDefined, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
        if(DEBUG) System.out.println(" in publishObjectClass() ");

        ObjectRecord objRecord = (ObjectRecord) objNaming.getObjectRecord(theClass);
        if(objRecord==null) throw new RTIcap.ObjectClassNotDefined();

        if(DEBUG) System.out.println(" Object Exists! () ");

        AttributeWatcher attWatcher ; 
        boolean except = false;
        for(int i=0;i<attributeList.length;i++)
        {
           attWatcher = (AttributeWatcher) objRecord.getAttributeWatcher(attributeList[i]);
           if(attWatcher!=null) attWatcher.register_publisher(_fedHandle,_fedId);
           else except = true;

        }// end of for

        if(except) throw new RTIcap.AttributeNotDefined();

        try
        {
           _fedHandle.startUpdates(theClass,attributeList);

        } catch(RTIcap.FederateInternalError fie) {}
          catch(RTIcap.ObjectClassNotPublished ocnp) {}
          catch(RTIcap.AttributeNotPublished anp) {}

        if(DEBUG) System.out.println(" out of publishObjectClass() ");

    }// end of publishObjectClass()

    /**
    * 5.3
    */
    public
    void unpublishObjectClass(short theClass, RTIcap.FederateAmbassador _fedHandle, 
                              short _fedId )
         throws RTIcap.ObjectClassNotDefined, RTIcap.FederateOwnsAttributes, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {

        ObjectRecord objRecord = (ObjectRecord) objNaming.getObjectRecord(theClass);
        if(objRecord==null) throw new RTIcap.ObjectClassNotDefined();

        objRecord.deregisterAll_publisher(_fedId);

    } 

    /**
    * 5.4
    */
    public
    void publishInteractionClass(short theInteraction,
                          RTIcap.FederateAmbassador _fedHandle, short _fedId   )
         throws RTIcap.InteractionClassNotDefined, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
       if(DEBUG) System.out.println(" in publishInteractionClass() ");
       InteractionRecord rec = interNaming.getInteractionRecord(theInteraction);
       
       rec.register_publisher(_fedHandle,_fedId);

       try
       {
          _fedHandle.startInteractionGeneration(theInteraction);
       } catch(RTIcap.FederateInternalError fie) {}
         catch(RTIcap.InteractionClassNotPublished icnp) {}

       if(DEBUG) System.out.println(" out of publishInteractionClass() ");

    }// end of publishInteractionClass()

    /**
    * 5.5
    */
    public
    void unpublishInteractionClass(short theInteraction, 
                           RTIcap.FederateAmbassador _fedHandle, short _fedId )
         throws RTIcap.InteractionClassNotDefined, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
       InteractionRecord rec = interNaming.getInteractionRecord(theInteraction);
       
       rec.deregister_publisher(_fedHandle,_fedId);

       try
       {
          _fedHandle.stopInteractionGeneration(theInteraction);
       } catch(RTIcap.FederateInternalError fie) {}
         catch(RTIcap.InteractionClassNotPublished icnp) {}

    }// end of unpublishInteractionClass()

    /**
    * 5.6
    *
    *  Get ObjectRecord
    *  Find AttributeWatcher Object
    *  Subscribe yourself to this queue.
    *
    */
    public
    void subscribeObjectClassAttribute1(short theClass, short[] attributeList,
                                       FederateAmbassador _fedHandle, short _fedId   )
         throws RTIcap.ObjectClassNotDefined, RTIcap.AttributeNotDefined, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {

        if(DEBUG) System.out.println(" in subscribeObjectClassAttribute() ");
        ObjectRecord objRecord = (ObjectRecord) objNaming.getObjectRecord(theClass);
        if(objRecord==null) 
        {   
            if(DEBUG) System.out.println(" ObjectClassNotDefined ");
            throw new RTIcap.ObjectClassNotDefined();
        }

        AttributeWatcher attWatcher ; 
        boolean except = false;

        for(int i=0;i<attributeList.length;i++)
        {
           attWatcher = (AttributeWatcher) objRecord.getAttributeWatcher(attributeList[i]);
           if(attWatcher!=null) attWatcher.subscribe(_fedHandle,_fedId);
           else except = true;

        }// end of for

        if(DEBUG) System.out.println(" out of subscribeObjectClassAttribute() ");
        if(except) throw new RTIcap.AttributeNotDefined();

    }// end of subscribeObjectClassAttribute()

    /**
    * 5.6
    */
    public
    void subscribeObjectClassAttribute2(short theClass, short theAttribute, 
                int theRegion, FederateAmbassador _fedHandle, short _fedId   )
         throws RTIcap.ObjectClassNotDefined, RTIcap.AttributeNotDefined, 
                RTIcap.RegionNotKnown, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
        throw new RTIcap.UnimplementedService();

    }// end of subscribeObjectClassAttribute2

    /**
    * 5.7
    */
    public
    void unsubscribeObjectClassAttribute(short theClass, short[] attributeList,
                                       FederateAmbassador _fedHandle, short _fedId   )
         throws RTIcap.ObjectClassNotDefined, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {

        ObjectRecord objRecord = (ObjectRecord) objNaming.getObjectRecord(theClass);
        if(objRecord==null) throw new RTIcap.ObjectClassNotDefined();

        AttributeWatcher attWatcher ; 
        boolean except = false;
        for(int i=0;i<attributeList.length;i++)
        {
           attWatcher = (AttributeWatcher) objRecord.getAttributeWatcher(attributeList[i]);
           if(attWatcher!=null) attWatcher.unsubscribe(_fedId);
           else except = true;

        }// end of for

    }// end of unsubscribeObjectClassAttribute()

    /**
    * 5.7
    */
    public
    void unsubscribeObjectClassAttribute1(short theClass,
                             FederateAmbassador _fedHandle, short _fedId )
         throws RTIcap.ObjectClassNotDefined, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
    }// end of unsubscribeObjectClassAttribute1()

    /**
    * 5.7
    */
    public
    void unsubscribeObjectClassAttribute2(short theClass, int theRegion,
                FederateAmbassador _fedHandle, short _fedId )
         throws RTIcap.ObjectClassNotDefined, RTIcap.RegionNotKnown, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
        throw new RTIcap.UnimplementedService();
    }// end of unsubscribeObjectClassAttribute2()

    /**
    * 5.8
    */
    public
    void subscribeInteractionClass1(short theClass, 
                FederateAmbassador _fedHandle, short _fedId   )
         throws RTIcap.InteractionClassNotDefined, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
       if(DEBUG) System.out.println(" in subscribeInteractionClass() ");

       InteractionRecord rec = interNaming.getInteractionRecord(theClass);
       
       if(rec!=null) rec.register_subscriber(_fedHandle,_fedId);
       else if(DEBUG) System.out.println(theClass+" does not exists ");

       if(DEBUG) System.out.println(" out of subscribeInteractionClass() ");
       
    }// end of subscribeInteractionClass()

    /**
    * 5.8
    */
    public
    void subscribeInteractionClass2(short theClass, int theRegion,
                FederateAmbassador _fedHandle, short _fedId   )
         throws RTIcap.InteractionClassNotDefined, RTIcap.RegionNotKnown, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }// end of subscribeInteractionClass2()

    /**
    * 5.9
    */
    public
    void unsubscribeInteractionClass1(short theClass, 
                FederateAmbassador _fedHandle, short _fedId   )
         throws RTIcap.InteractionClassNotDefined, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
       InteractionRecord rec = interNaming.getInteractionRecord(theClass);
       
       rec.deregister_subscriber(_fedHandle,_fedId);

    }// end of unsubscribeInteractionClass()

    /**
    * 5.9
    */
    public
    void unsubscribeInteractionClass2(short theClass, int theRegion,
                FederateAmbassador _fedHandle, short _fedId   )
         throws RTIcap.InteractionClassNotDefined, RTIcap.RegionNotKnown, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
        throw new RTIcap.UnimplementedService();

    }// end of unsubscribeInteractionClass2()


}// end of DeclarationManager

